The implementations of the workloads in this directory are intended
to be compared against those released as part of the
[ParallelAccelerator.jl](https://github.com/IntelLabs/ParallelAccelerator.jl)
package.  See the `examples` directory in the ParallelAccelerator.jl
repository for implementations of each workload in Julia, with and
without the use of ParallelAccelerator.

## Organization

Each subdirectory in this directory contains one or more versions of a
workload (typically MATLAB, but we also have Python or C/C++
implementations of a few workloads).

For a few workloads, we also have alternate "devectorized"
implementations in Julia, or alternate Julia implementations that use
`@threads` (explicit threading) instead of using ParallelAccelerator.

Each directory has a `src`
subdirectory, containing the actual implementations, and a
`benchmarking` subdirectory, containing benchmarking scripts.

See the READMEs in individual directories for descriptions of the
workloads.

The `util/` directory contains utility scripts for benchmarking (and
plotting benchmarking results for) the code in this directory.
Running something like

```
util/run-benchmarks.sh util/benchmark-list.txt
```

should build and run everything (note that the full suite of
benchmarks takes some time to run).

## Running individual workloads

To run individual implementations of workloads, the
implementation-specific scripts in the `benchmarking/` directory for
each workload are useful.

To run all implementations of a particular workload and log the
results, do something like:

    cd benchmarking/
    source ../../util/env-vars.sh
    ./benchmark.sh 1> totaltimes.log 2> offloadtimes.log

This will generate two log files, which you can process using
`munge-results.py` in the `util/` directory:

    ../../util/munge-results.py totaltimes.log offloadtimes.log

This produce a text summary of the benchmark results as well
as a PDF figure generated using matplotlib.
