# Manually devectorized version of blackscholes.jl, contributed by
# Jiahao Chen

cndf2(in) = 0.5 .+ 0.5 .* erf(0.707106781 .* in)

function blackscholes1{T<:Real}(sptprice::Vector{T},
                               strike::Vector{T},
                               rate::Vector{T},
                               volatility::Vector{T},
                               time::Vector{T})
    n = length(sptprice)
    put = Array(T, n)
    for i = 1:n
        logterm = log10(sptprice[i] / strike[i])
        powterm = 0.5 * volatility[i]^2
        den = volatility[i] * √time[i]
        d1 = (((rate[i] + powterm) * time[i]) + logterm) / den
        d2 = d1 - den
        NofXd1 = cndf2(d1)
        NofXd2 = cndf2(d2)
        futureValue = strike[i] * exp(-rate[i]*time[i])
        c1 = futureValue * NofXd2
        call = sptprice[i] * NofXd1 - c1
        put[i] = call - futureValue + sptprice[i]
    end
    return put
end

function run(iterations) 
    sptprice   = Float64[ 42.0 for i=1:iterations ]
    initStrike = Float64[ 40.0 + (i / iterations) for i=1:iterations ]
    rate       = Float64[ 0.5 for i=1:iterations ]
    volatility = Float64[ 0.2 for i=1:iterations ]
    time       = Float64[ 0.5 for i=1:iterations ]

    tic()
    put = blackscholes1(sptprice, initStrike, rate, volatility, time)
    t = toc()
    println("checksum: ", sum(put))
    return t
end

srand(0)

iterations = 100000000

if (length(ARGS) > 0)
    iterations = parseint(Int, ARGS[1])
end 
if (length(ARGS) > 1)
    println("Too many arguments to blackscholes.")
    exit(-1)
end

println("iterations  = ",iterations)

tic()
blackscholes1(Float64[],Float64[],Float64[],Float64[],Float64[])
println("SELFPRIMED ", toq())

time = run(iterations)
println("Rate = ", iterations / time, " Opts/sec")
println("SELFTIMED ", time)
