# Julia set computation

This code is ported from a MATLAB version from http://www.albertostrumia.it/Fractals/FractalMatlab/Jul.html .  Specifically, it's ported from http://www.albertostrumia.it/Fractals/FractalMatlab/Julia/juliaSH.m, which uses `c = -.745429 + .11308*i` as the constant.  (The choice of the complex constant `c` is what determines what a given Julia set will look like -- there is a different Julia set for each `c`.)

The demo calculates a sequence of ten progressively magnified images.

## Implementations

  * js.m: The original MATLAB code from http://www.albertostrumia.it/Fractals/FractalMatlab/Julia/juliaSH.m .
  * See the examples directory in the ParallelAccelerator package for the `@acc` and plain Julia versions.  The plain Julia version is a verbatim port of the original MATLAB code (but with an unnecessary assignment in the inner loop (in `iterate`) removed).
  * js-arraycomprehension.jl: Much faster plain Julia implementation that uses an array comprehension to compute each value of the updated array.

## Notes

This example uses a very small number of operators in the core loop (Z is a complex dense matrix, c is a complex number, W is the color map):

```
for k=1:col
    Z=Z.^2.+c
    W=(exp(-abs(Z)))
end
```
The interesting tidbit is that it represents points outside of the set via overflow to the NaN value.
