# Run using something like:
# ./benchmark.sh 1> totaltimes.log 2> offloadtimes.log

# This will produce two files that can be processed with
# util/munge-results.py:

# ../../util/munge-results.py totaltimes.log offloadtimes.log

NUM_TRIALS=5

echo "BENCHMARK NAME: Laplace 3D 6-point stencil" >&2
echo "BENCHMARK DESCRIPTION: 290x290x290 input array, 1000 iterations" >&2
echo "BENCHMARK SHORT NAME: laplace-3d" >&2

echo "Version: C (optimized SSE, 36 threads):" >&2
for ((i=1; i<=NUM_TRIALS; i++)); do OMP_NUM_THREADS=36 ./runc-ninja-sse.sh; done

echo "Version: C (optimized SSE, 18 threads):" >&2
for ((i=1; i<=NUM_TRIALS; i++)); do OMP_NUM_THREADS=18 ./runc-ninja-sse.sh; done

echo "Version: C (optimized AVX, 36 threads):" >&2
for ((i=1; i<=NUM_TRIALS; i++)); do OMP_NUM_THREADS=36 ./runc-ninja-avx.sh; done

echo "Version: C (optimized AVX, 18 threads):" >&2
for ((i=1; i<=NUM_TRIALS; i++)); do OMP_NUM_THREADS=18 ./runc-ninja-avx.sh; done

echo "Version: C (simple parallel, 36 threads):" >&2
for ((i=1; i<=NUM_TRIALS; i++)); do OMP_NUM_THREADS=36 ./runc-naive-parallel.sh; done

echo "Version: C (simple parallel, 18 threads):" >&2
for ((i=1; i<=NUM_TRIALS; i++)); do OMP_NUM_THREADS=18 ./runc-naive-parallel.sh; done

echo "Version: C (simple serial, 1 thread):" >&2
for ((i=1; i<=NUM_TRIALS; i++)); do OMP_NUM_THREADS=1 ./runc-orig.sh; done

echo "Version: @acc (36 threads):" >&2
for ((i=1; i<=NUM_TRIALS; i++)); do OMP_NUM_THREADS=36 ./runjulia5-acc.sh; done

echo "Version: @acc (18 threads):" >&2
for ((i=1; i<=NUM_TRIALS; i++)); do OMP_NUM_THREADS=18 ./runjulia5-acc.sh; done

echo "Version: @acc (1 thread):" >&2
for ((i=1; i<=NUM_TRIALS; i++)); do OMP_NUM_THREADS=1 ./runjulia5-acc.sh; done

echo "Version: Julia 0.5 with @threads (36 threads):" >&2
for ((i=1; i<=NUM_TRIALS; i++)); do MKL_NUM_THREADS=1 MKL_DYNAMIC=false JULIA_NUM_THREADS=36 ./runjulia5-threads.sh; done

echo "Version: Julia 0.5 with @threads (18 threads):" >&2
for ((i=1; i<=NUM_TRIALS; i++)); do MKL_NUM_THREADS=1 MKL_DYNAMIC=false JULIA_NUM_THREADS=18 ./runjulia5-threads.sh; done

echo "Version: Julia 0.5 with @threads (1 thread):" >&2
for ((i=1; i<=NUM_TRIALS; i++)); do MKL_NUM_THREADS=1 MKL_DYNAMIC=false JULIA_NUM_THREADS=1 ./runjulia5-threads.sh; done

echo "Version: Julia 0.5 (1 thread):" >&2
for ((i=1; i<=NUM_TRIALS; i++)); do ./runjulia5.sh; done

echo "Version: Matlab:" >&2
for ((i=1; i<=NUM_TRIALS; i++)); do ./runmatlab.sh; done

echo "Version: Matlab (1 thread):" >&2
for ((i=1; i<=NUM_TRIALS; i++)); do ./runmatlab-single.sh; done
