#include <unistd.h>
#include <iostream>
#include <fstream>
#include <string>
#include <cstring>
#include <assert.h>
#include <cstdlib>
#include <fstream>
#include "mkl_cblas.h"
#include <cmath>
#include <xmmintrin.h>
#include "omp.h"
#include "timestamp.h"
#include "barrier.h"

#define MAX_COLS 60
int makecolorwheel(int colorwheel[][3], int maxcols);
int colorwheel[MAX_COLS][3];
float maxrad;
int ncols;

int nthreads;
static TREE_BARRIER_TYPE tree_barrier;
#define __BARRIER__ TREE_BARRIER(&tree_barrier, tid, nthreads);


#ifdef __MIC__
#define CPU_FREQ (1.09e9)
#else
#define CPU_FREQ (2.7e9)
#endif

#define C2F(x) ((float)(x)/255.0f)
float *Ix, *Iy, *It, *r, *x, *z, *p, *Ap;
float s = 0.0f;

void checksum(const float *flow, int width, int height, int pitch)
{
  float sum_u = 0, sum_v = 0;
  for (int y = 0; y < height; y++)
    for (int x = 0; x < width; x++) {
      sum_u += flow[x + y * pitch];
      sum_v += flow[x + y * pitch + pitch * height];
    }
  printf("checksum: %f %f\n", sum_u, sum_v);
}


float interp(const float* im, float x, float y, int width, int height, int pitch) {
  float res = 0.0f;
  int xx = (int)x;
  int yy = (int)y;
  float alpha = x - xx;
  float beta = y - yy;
  int pp = yy * pitch + xx;
  res = (1.0f - beta) * (alpha * (im[pp+1]) + (1.0f - alpha) * (im[pp]) ) + (beta) * (alpha * (im[pp+1+pitch]) + (1.0f- alpha) * (im[pp+pitch]) );
  return (res);
}


void downsample(const float* orig_image, int iwidth, int iheight, int ipitch, float* small_image, int owidth, int oheight, int opitch) {
  if (iwidth == owidth && iheight == oheight) {
    memcpy(small_image, orig_image, sizeof(float)*ipitch*iheight);
    return;
  }

  float factorX = (float)iwidth/(float)owidth;
  float factorY = (float)iheight/(float)oheight;
 #if 1
  #pragma omp parallel for //num_threads(nthreads)
  //for (int pp = 0; pp < oheight*owidth; pp++) {
  for (int y = 0; y < oheight; y++ ) {
    #pragma simd
    for (int x = 0; x < owidth; x++) {
        //int x = pp%owidth;
        //int y = pp/owidth;
        float result = 0.0f;
        float fi = ((float)y)*factorY;
        int i = floorf(fi);
        float fLastI = ((float)(y+1))*factorY;
        int LastI = floorf(fLastI);
        do
        {
            float fj = ((float)x)*factorX;
            int j = floorf(fj);
            float fLastJ = ((float)(x+1))*factorX;
            int LastJ = floorf(fLastJ);
            do
            {
                float coeff = 1.0;
                if(fi-i >0)
                {
                    coeff = (1-fi+i);
                }
                if(fj-j > 0)
                {
                    coeff *= (1-fj+j);
                }
                if(i==LastI)
                {
                    coeff *= (fLastI-LastI);
                }
                if(j==LastJ)
                {
                    coeff *= (fLastJ-LastJ);
                }

                result += coeff*orig_image[i*ipitch+j];
                j++;

            }while(j<=LastJ && j<iwidth);
            i++;
        } while(i<=LastI && i<iheight);
      small_image[y * opitch + x] = result/(factorX*factorY);
    }
  }
  #else
  float fxfy = 1.0f/(factorX*factorY);
  #pragma omp parallel for 
  for (int iy = 0; iy < iheight; iy++){
    #pragma simd
    for (int ix = 0; ix < iwidth; ix++) {
      int sx = (int)((float)ix/factorX);
      int ex = sx + 1;
      int sy = (int)((float)iy/factorY);
      int ey = sy + 1;
      float exfx = ex*factorX;
      float eyfy = ey*factorY;
      float input_value = fxfy*orig_image[iy*ipitch + ix];
      if (exfx >= ix+1) {
        if (eyfy >= iy+1) {
          small_image[sy*opitch + sx] += input_value;
        } else {
          small_image[sy*opitch + sx] += (eyfy-iy)*input_value;
          small_image[ey*opitch + sx] += (1.0-eyfy+iy)*input_value;
        }
      } else {
        if (eyfy >= iy+1) {
          small_image[sy*opitch + sx] += (exfx-ix)*input_value;
          small_image[sy*opitch + ex] += (1.0-exfx+ix)*input_value;
        } else {
          small_image[sy*opitch + sx] += (eyfy-iy)*(exfx-ix)*input_value;
          small_image[sy*opitch + ex] += (eyfy-iy)*(1.0-exfx+ix)*input_value;
          small_image[ey*opitch + sx] += (1.0-eyfy+iy)*(exfx-ix)*input_value;
          small_image[ey*opitch + ex] += (1.0-eyfy+iy)*(1.0-exfx+ix)*input_value;
        }
      }
    }
  }
  #endif


}

void interpolate(const float* input_flow, float* output_flow, int iwidth, int iheight, int ipitch, int owidth, int oheight, int opitch) {
  assert(owidth >= iwidth);
  if (owidth == iwidth && oheight == iheight) {
    memcpy(output_flow, input_flow, ipitch*iheight*2*sizeof(float));
    return;
  }

  float scale = (double)owidth/(double)iwidth;
  int inpixelspitch = ipitch*iheight;
  int onpixelspitch = opitch*oheight;

  #pragma omp parallel for num_threads(nthreads)
  for (int i = 0; i < oheight; i++ ) {
    #pragma simd
    for (int j = 0; j < owidth; j++) {
      float x = (float)j/(float)owidth*(float)iwidth;
      float y = (float)i/(float)oheight*(float)iheight;
      
      float res = interp(input_flow, x, y, iwidth, iheight, ipitch); 
      output_flow[i * opitch + j] = res/scale;

      float res2 = interp(input_flow + inpixelspitch, x, y, iwidth, iheight, ipitch); 
      output_flow[i * opitch + j + onpixelspitch] = res2/scale;
    }
  }
}

void warp_motion(const float* orig_image, const float* flow, float* output_image, const float* other_image, int width, int height, int pitch) {
  int npixelspitch = pitch*height;

  #pragma omp parallel for num_threads(nthreads)
  for (int i = 0; i < height; i++) {
    #pragma simd
    for (int j = 0; j < width; j++) {
      int pp = i * pitch + j;
      float u = j + flow[pp];
      float v = i + flow[pp+npixelspitch];
      float res;
      if (u < 0 || u > width-1 || v < 0 || v > height-1) {
        res = other_image[pp];
      } else {
        res = interp(orig_image, u, v, width, height, pitch);
      }
      output_image[pp] = res;
    }
  }
}

void blockJacobiPreconditioner(const float* Ix, const float* Iy, float* y, const float* x, float lam, int width, int height, int pitch, int hbegin, int hend) { // y = M^-1 x
  int npixelspitch = pitch*height;

  for (int i = hbegin; i < hend; i++) {
    #pragma simd
    for (int j = 0; j < width; j++) {
      float ures, vres;
      int pp = i * pitch + j;
      float local_u = x[pp];
      float local_v = x[pp + npixelspitch];

      float a,b,c;
      a = Ix[pp]*Ix[pp] + 4*lam;
      b = Ix[pp]*Iy[pp];
      c = Iy[pp]*Iy[pp] + 4*lam;
      float invdet = 1.0f/(a*c - b*b);

      ures = (c*local_u - b*local_v)*invdet;
      vres = (a*local_v - b*local_u)*invdet;

      y[pp] = ures;
      y[pp+npixelspitch] = vres;
    }
  }

}

void mvm (const float* Ix, const float* Iy, float* y, const float* x, float lam, int width, int height, int pitch, int hbegin, int hend) {

#define PREFETCH_DIST (3)

  int npixelspitch = pitch*height;
  for (int i = hbegin; i < hend; i++) {
    #pragma simd
    for (int j = 0; j < width; j++) {
      float ures, vres;
      int pp = i * pitch + j;
      float local_u = x[pp];
      float local_v = x[pp + npixelspitch];
      //_mm_prefetch((char*)(x+pp+PREFETCH_DIST*16), _MM_HINT_T0);      
      //_mm_prefetch((char*)(x+pp+npixelspitch+PREFETCH_DIST*16), _MM_HINT_T0);      

      ures = Ix[pp]* (Ix[pp]*local_u + Iy[pp]* local_v) + 4.0f*lam*local_u;
      vres = Iy[pp]* (Ix[pp]*local_u + Iy[pp]* local_v) + 4.0f*lam*local_v;

      if (i > 0) {
        ures += -lam * x[pp-pitch];
        vres += -lam * x[pp-pitch + npixelspitch];
      }
      if (j > 0) {
        ures += -lam * x[pp-1];
        vres += -lam * x[pp-1 + npixelspitch];
      }
      if (j < width-1) {
        ures += -lam * x[pp+1];
        vres += -lam * x[pp+1 + npixelspitch];
      }
      if (i < height-1) {
        ures += -lam * x[pp+pitch];
        vres += -lam * x[pp+pitch + npixelspitch];
      }

      y[pp] = ures;
      y[pp+npixelspitch] = vres;
    }
  }

}


void my_sdot(int n, float* x, float* y, int tid, float* res ) {

  if (tid == 0) s = 0.0f;

  #pragma omp barrier

  #pragma omp for reduction(+:s)
  for (int i = 0; i < n; i++) {
     s += x[i]*y[i];
  }

  if (tid == 0) res[0] = s;
  #pragma omp barrier
}

void single_scale_of_old(const float* im1, const float* im2, float* x, int num_iterations, float lam, int width, int height, int pitch)
{
  double mem_ts = timestamp();

  Ix = new float[pitch*height];
  Iy = new float[pitch*height];
  It = new float[pitch*height];
  r = new float[2*pitch*height];
  z = new float[2*pitch*height];
  p = new float[2*pitch*height];
  Ap = new float[2*pitch*height];
  float* t = new float[2*pitch*height];

  //x = 0
  memset(x, 0, sizeof(float)*2*pitch*height);
  
  memset(Ix, 0, sizeof(float)*pitch*height);
  memset(Iy, 0, sizeof(float)*pitch*height);
  memset(It, 0, sizeof(float)*pitch*height);
  memset(p, 0, sizeof(float)*2*pitch*height);
  memset(Ap, 0, sizeof(float)*2*pitch*height);
  memset(z, 0, sizeof(float)*2*pitch*height);
  memset(t, 0, sizeof(float)*2*pitch*height);
  memset(r, 0, sizeof(float)*2*pitch*height);

  
  int npixelspitch = pitch*height;

  //Ix, Iy, It
  //Ix2, Iy2, IxIy, IxIt, IyIt 
  float alpha, beta, rsold, rsnew, pTAp;

  #pragma omp parallel shared(s, alpha, beta, rsold, rsnew, pTAp, nthreads)
  {
  
   int tid = omp_get_thread_num();
   int num_threads = omp_get_num_threads();
   if (tid == 0) printf("num_threads = %d\n", num_threads);
   if (tid == 0) nthreads = num_threads;
   int h_per_thread = (height+num_threads-1)/num_threads;
   int hbegin = tid*h_per_thread;
   int hend = hbegin + h_per_thread;
   if (hbegin >= height) hbegin = height;
   if (hend >= height) hend = height;
  
   unsigned long long int start, time;
   time = 0;
   start = __rdtsc();

  for (int i = hbegin; i < hend; i++) {
    #pragma simd
    for (int j = 0; j < width; j++) {
      int pp = i * pitch + j;
      Ix[pp] = (j >1 && j < width-2)? (-1.0f*(im1[pp+2])-8.0f*(im1[pp+1])+8.0f*(im1[pp-1])+1.0f*(im1[pp-2]))/12.0f: 0.0f;
      Iy[pp] = (i >1 && i < height-2)? (-1.0f*(im1[pp+2*pitch])-8.0f*(im1[pp+pitch])+8.0f*(im1[pp-pitch])+1.0f*(im1[pp-2*pitch]))/12.0f: 0.0f;
      It[pp] = (im1[pp]) - (im2[pp]);
    }
  }

  time = __rdtsc() - start;
  if(tid == 0) printf("Gradient BW = %.3f GBps\n", (8*width*height*sizeof(float))/((double)(time)/CPU_FREQ)/1e9);
  time = 0;
  
  // t = A*x
  //mvm(Ix, Iy, t, x, lam, width, height, pitch);
  // t is set to 0

  // r = b - t
  for (int i = pitch*hbegin; i < pitch*hend; i++) {
    r[i] = -Ix[i]*It[i];// - t[i];
    r[i + npixelspitch] = -Iy[i]*It[i];// - t[i + npixelspitch];
  }

  //z = M^-1 r
  blockJacobiPreconditioner(Ix, Iy, z, r, lam, width, height, pitch, hbegin, hend);

  // p = z
  if(hend>hbegin) memcpy(p + hbegin*pitch*2, z + hbegin*pitch*2, sizeof(float)*(hend-hbegin)*pitch*2);

  // rsold = dot(r.T,z)
  my_sdot(2*npixelspitch, r, z, tid, &rsold);

  double ts = timestamp();

  for(int iter = 0 ; iter < num_iterations; iter++)
  {
    start = __rdtsc();
    // Ap = A * p
    mvm(Ix, Iy, Ap, p, lam, width, height, pitch, hbegin, hend);
    time += __rdtsc() - start;

     // pTAp = dot(p.T, Ap)
    my_sdot(2*npixelspitch, p, Ap, tid, &pTAp);

    //float alpha = rsold / pTAp; // kernel for this (alpha1)
    alpha = rsold/pTAp;

    // x = x + alpha * p
    // r = r - alpha * Ap
    #pragma omp for 
    for (int i = 0; i < 2*npixelspitch; i++) {
      x[i] += alpha*p[i];
      r[i] += -alpha*Ap[i];
    }
    #pragma omp barrier

    //z = M^-1 r
    blockJacobiPreconditioner(Ix, Iy, z, r, lam, width, height, pitch, hbegin, hend);

    // rsnew = dot(r.T, z)
    //float beta = rsnew/rsold
    my_sdot(2*npixelspitch, r, z, tid, &rsnew);
    beta = rsnew/rsold;

    // p = z + beta * p
    #pragma omp for
    for (int i = 0; i < 2*npixelspitch; i++) {
      p[i] = beta*p[i] + z[i];
    }
    #pragma omp barrier

    // rsold = rsnew
    rsold = rsnew;

  }
  if(tid == 0) printf("MVM BW = %.3f GBps = %.3f ms\n", (8*width*height*sizeof(float))/((double)(time)/CPU_FREQ)/1e9*num_iterations,(double)time/CPU_FREQ*1e3/num_iterations );
  if (tid == 0) printf("BLAS1 BW = %.3f GBps\n", (23*width*height*sizeof(float))/(timestamp() - ts)/1e9*num_iterations);
  }

}

// p = beta*p + z
// Ap = A * p
// pTAp = dot(p, Ap)
void pcg_merge_1(float* p, float* z, float* Ap, float* Ix, float* Iy, float lam, float beta, int width, int height, int pitch) {

  int npixelspitch = pitch*height;
  int tid = omp_get_thread_num();

  #pragma omp for 
  for (int i = 0; i < height; i++) {
    #pragma simd
    for (int j = 0; j < width; j++) {
      int pp = i * pitch + j;
      p[pp] = beta * p[pp] + z[pp];
      p[pp + npixelspitch] = beta * p[pp+ npixelspitch] + z[pp+ npixelspitch];
    } 
  }
  //#pragma omp barrier
  __BARRIER__;

  #pragma omp for reduction(+:s)
  for (int i = 0; i < height; i++) {
    #pragma simd
    for (int j = 0; j < width; j++) {
      float ures, vres;
      int pp = i * pitch + j;
      float local_u = p[pp];
      float local_v = p[pp + npixelspitch];
      //_mm_prefetch((char*)(x+pp+PREFETCH_DIST*16), _MM_HINT_T0);      
      //_mm_prefetch((char*)(x+pp+npixelspitch+PREFETCH_DIST*16), _MM_HINT_T0);      

      ures = Ix[pp]* (Ix[pp]*local_u + Iy[pp]* local_v) + 4.0f*lam*local_u;
      vres = Iy[pp]* (Ix[pp]*local_u + Iy[pp]* local_v) + 4.0f*lam*local_v;

      if (i > 0) {
        ures += -lam * p[pp-pitch];
        vres += -lam * p[pp-pitch + npixelspitch];
      }
      if (j > 0) {
        ures += -lam * p[pp-1];
        vres += -lam * p[pp-1 + npixelspitch];
      }
      if (j < width-1) {
        ures += -lam * p[pp+1];
        vres += -lam * p[pp+1 + npixelspitch];
      }
      if (i < height-1) {
        ures += -lam * p[pp+pitch];
        vres += -lam * p[pp+pitch + npixelspitch];
      }

      Ap[pp] = ures;
      Ap[pp+npixelspitch] = vres;
      s += local_u*ures + local_v*vres;
    }
  }
 
  //if (tid == 0) s = 0.0f;

  //#pragma omp barrier

  //#pragma omp for reduction(+:s)
  //for (int i = 0; i < n; i++) {
  //   s += x[i]*y[i];
  //}

  //if (tid == 0) res[0] = s;
  //#pragma omp barrier
 
}

// x = x + alpha*p
// r = r - alpha*Ap
// z = M^-1 * r
// rsnew = dot(z, r)
void pcg_merge_2(float* x, float* r, float* p, float* Ap, float* z, float* Ix, float* Iy, float lam, float alpha, int width, int height, int pitch) {
  
  int npixelspitch = pitch*height;
  #pragma omp for reduction(+:s)
  for (int i = 0; i < height; i++) {
    #pragma simd
    for (int j = 0; j < width; j++) {
      int pp = i * pitch + j;
      x[pp] += alpha * p[pp];
      x[pp + npixelspitch] += alpha * p[pp + npixelspitch];
      
      r[pp] -= alpha * Ap[pp];
      r[pp + npixelspitch] -= alpha * Ap[pp + npixelspitch];

      float ures, vres;
      float local_u = r[pp];
      float local_v = r[pp + npixelspitch];

      float a,b,c;
      a = Ix[pp]*Ix[pp] + 4*lam;
      b = Ix[pp]*Iy[pp];
      c = Iy[pp]*Iy[pp] + 4*lam;
      float invdet = 1.0f/(a*c - b*b);

      ures = (c*local_u - b*local_v)*invdet;
      vres = (a*local_v - b*local_u)*invdet;

      z[pp] = ures;
      z[pp+npixelspitch] = vres;

      s += local_u*ures + local_v*vres;

    }
  }
  
}

void pcg_init(float* r, float* z, float* Ix, float* Iy, float* It, float lam, int width, int height, int pitch) {

  int npixelspitch = pitch*height;
  #pragma omp for reduction(+:s)
  for (int i = 0; i < height; i++) {
    #pragma simd
    for (int j = 0; j < width; j++) {
      int pp = i * pitch + j;
      
      r[pp] = -Ix[pp]*It[pp];
      r[pp + npixelspitch] = -Iy[pp]*It[pp];

      float ures, vres;
      float local_u = r[pp];
      float local_v = r[pp + npixelspitch];

      float a,b,c;
      a = Ix[pp]*Ix[pp] + 4*lam;
      b = Ix[pp]*Iy[pp];
      c = Iy[pp]*Iy[pp] + 4*lam;
      float invdet = 1.0f/(a*c - b*b);

      ures = (c*local_u - b*local_v)*invdet;
      vres = (a*local_v - b*local_u)*invdet;

      z[pp] = ures;
      z[pp+npixelspitch] = vres;

      s += local_u*ures + local_v*vres;

    }
  }
}


void single_scale_of(const float* im1, const float* im2, float* x, int num_iterations, float lam, int width, int height, int pitch)
{
  double mem_ts = timestamp();

  //float* t = new float[2*pitch*height];

  //x = 0
  memset(x, 0, sizeof(float)*2*pitch*height);
  
  //memset(Ix, 0, sizeof(float)*pitch*height);
  //memset(Iy, 0, sizeof(float)*pitch*height);
  //memset(It, 0, sizeof(float)*pitch*height);
  memset(p, 0, sizeof(float)*2*pitch*height);
  //memset(Ap, 0, sizeof(float)*2*pitch*height);
  //memset(z, 0, sizeof(float)*2*pitch*height);
  //memset(t, 0, sizeof(float)*2*pitch*height);
  //memset(r, 0, sizeof(float)*2*pitch*height);
  
  //printf("Malloc + memset time = %f \n", timestamp()-mem_ts);

  int npixelspitch = pitch*height;

  //Ix, Iy, It
  //Ix2, Iy2, IxIy, IxIt, IyIt 
  float alpha, beta, rsold, rsnew, pTAp;
  beta = 0.0f;
  alpha = 0.0f;
  //if (height <= 50) nthreads = 1;
  //int nt = (240 <= height/10)?(240):(height/10);
  #pragma omp parallel shared(s, alpha, beta, rsold, rsnew, pTAp, nthreads) num_threads(nthreads) //if(height>50)
  {
  
   int tid = omp_get_thread_num();
   int num_threads = omp_get_num_threads();
   if (tid == 0) printf("num_threads = %d\n", num_threads);
   //if (tid == 0) nthreads = num_threads;
   int h_per_thread = (height+num_threads-1)/num_threads;
   int hbegin = tid*h_per_thread;
   int hend = hbegin + h_per_thread;
   if (hbegin >= height) hbegin = height;
   if (hend >= height) hend = height;
  
   unsigned long long int start, time;
   time = 0;
   start = __rdtsc();

  for (int i = hbegin; i < hend; i++) {
  //for (int i = 0; i < height; i++) {
    #pragma simd
    for (int j = 0; j < width; j++) {
      int pp = i * pitch + j;
      Ix[pp] = (j >1 && j < width-2)? (-1.0f*(im1[pp+2])-8.0f*(im1[pp+1])+8.0f*(im1[pp-1])+1.0f*(im1[pp-2]))/12.0f: 0.0f;
      Iy[pp] = (i >1 && i < height-2)? (-1.0f*(im1[pp+2*pitch])-8.0f*(im1[pp+pitch])+8.0f*(im1[pp-pitch])+1.0f*(im1[pp-2*pitch]))/12.0f: 0.0f;
      It[pp] = (im1[pp]) - (im2[pp]);
    }
  }

  time = __rdtsc() - start;
  if(tid == 0) printf("Gradient BW = %.3f GBps\n", (8*width*height*sizeof(float))/((double)(time)/CPU_FREQ)/1e9);
  time = 0;
  
  if (tid == 0) s = 0.0f;
  #pragma omp barrier
  pcg_init(r, z, Ix, Iy, It, lam, width, height, pitch);
  if (tid == 0) rsold = s;
  #pragma omp barrier

  double ts = timestamp();

  for(int iter = 0 ; iter < num_iterations; iter++)
  {

    // p = z + beta * p
    /*#pragma omp for
    for (int i = 0; i < 2*npixelspitch; i++) {
      p[i] = beta*p[i] + z[i];
    }

    start = __rdtsc();
   // Ap = A * p
    mvm(Ix, Iy, Ap, p, lam, width, height, pitch, hbegin, hend);
    time += __rdtsc() - start;

     // pTAp = dot(p.T, Ap)
    my_sdot(2*npixelspitch, p, Ap, tid, &pTAp);*/

    if (tid == 0) s = 0.0f;
    #pragma omp barrier
    //__BARRIER__;
    //start = __rdtsc();
    pcg_merge_1(p, z, Ap, Ix, Iy, lam, beta, width, height, pitch);
    if (tid == 0) {
	pTAp = s;
    	alpha = rsold/pTAp;
        //printf("%lld cycles \n", __rdtsc()-start);
    }
    #pragma omp barrier
    //__BARRIER__;
    

    //float alpha = rsold / pTAp; // kernel for this (alpha1)
    //alpha = rsold/pTAp;

/*
    // x = x + alpha * p
    // r = r - alpha * Ap
    #pragma omp for 
    for (int i = 0; i < 2*npixelspitch; i++) {
      x[i] += alpha*p[i];
      r[i] += -alpha*Ap[i];
    }
    #pragma omp barrier

    //z = M^-1 r
    blockJacobiPreconditioner(Ix, Iy, z, r, lam, width, height, pitch, hbegin, hend);

    // rsnew = dot(r.T, z)
    //float beta = rsnew/rsold
    my_sdot(2*npixelspitch, r, z, tid, &rsnew);*/

    if (tid == 0) s = 0.0f;
    #pragma omp barrier
    //__BARRIER__;
    pcg_merge_2(x, r, p, Ap, z, Ix, Iy, lam, alpha, width, height, pitch);
    if (tid == 0) {
	rsnew = s;
    	beta = rsnew/rsold;
    	rsold = rsnew;
    }
    #pragma omp barrier
    //__BARRIER__;
   

    //beta = rsnew/rsold;

    //#pragma omp barrier

    // rsold = rsnew
    //rsold = rsnew;

  }
  //if(tid == 0) printf("MVM BW = %.3f GBps = %.3f ms\n", (8*width*height*sizeof(float))/((double)(time)/CPU_FREQ)/1e9*num_iterations,(double)time/CPU_FREQ*1e3/num_iterations );
  if (tid == 0) printf("BLAS1+MVM BW = %.3f GBps\n", ((30)*width*height*sizeof(float))/(timestamp() - ts)/1e9*num_iterations);
  }
}


void optical_flow2(const float* im1, const float* im2, float* x, int num_iterations, float lam, int width, int height, int pitch, int nscales = 1) {
  //float diff = 0.0f;
  //for (int  j = 0; j < height; j++) for (int k = 0; k < width; k++) diff += fabs(im2[j*pitch+k] - im1[j*pitch+k]);
  //printf("Diff = %f \n", diff/(width*height));

  if (nscales == 1) {
    single_scale_of(im1, im2, x, num_iterations, lam, width, height, pitch);

  } else {


  //calculate sizes for nscales.
  //alloc memory
  //min dimension = 50 pixels,
  float scale = pow((double)width/50.0, -1.0/nscales);

  float* first_small_image = new float[pitch*height];
  float* second_small_image = new float[pitch*height];
  float* warped_image = new float[pitch*height];
  float* x_old = new float[2*pitch*height];
  float* x_new = new float[2*pitch*height];
  float* dx = new float[2*pitch*height];
  memset(dx, 0, 2*pitch*height*sizeof(float));
  memset(x_old, 0, 2*pitch*height*sizeof(float));
  memset(x_new, 0, 2*pitch*height*sizeof(float));

  int iwidth, iheight, ipitch;
  double ssof_time = 0.0;
  char* str_num_threads = getenv("OMP_NUM_THREADS");
  int max_threads;
  if (str_num_threads != NULL) {
    max_threads = atoi(str_num_threads);
  } else {
    max_threads = omp_get_max_threads();
  }
  printf("OMP_NUM_THREADS = %d \n", max_threads);

  for (int i = nscales; i >= 0; i--) {
   
    //new sizes
    int owidth, oheight, opitch;
    owidth = (int)(width * pow(scale, i));
    oheight = (int)(height * pow(scale, i));
    opitch = ((owidth+31)/32)*32;
    printf("Scale = %f width, height, pitch = %d, %d, %d\n", pow(scale, i), owidth, oheight, opitch);
    nthreads = (max_threads <= oheight/10)?(max_threads):(oheight/10);
    
    double ts1 = timestamp();
    //downsample im1,im2
    downsample(im1, width, height, pitch, first_small_image, owidth, oheight, opitch);
    downsample(im2, width, height, pitch, second_small_image, owidth, oheight, opitch);
    
    if (i < nscales) {
      //upsample flow
      interpolate(x_old, x_new, iwidth, iheight, ipitch, owidth, oheight, opitch);
      //interpolate im1 using flow
      warp_motion(second_small_image, x_new, warped_image, first_small_image, owidth, oheight, opitch);
    } else {
      memcpy(warped_image, second_small_image, opitch*oheight*sizeof(float));
    }
   
    if (i == 0) num_iterations *= 5;


    double ts = timestamp();
    //single scale solve to get du,dv
    single_scale_of(first_small_image, warped_image, dx, num_iterations, lam, owidth, oheight, opitch);
    double end_ts = timestamp(); 
    //printf("Single scale of time = %f seconds \n", end_ts - ts);
    ssof_time += end_ts - ts;
    
    //u=u+du
    //#pragma omp parallel for num_threads(nthreads)
    //for (int j = 0; j < 2*opitch*oheight; j++) x_new[j] += dx[j];
    #pragma omp parallel for num_threads(nthreads)
    for (int y = 0; y < 2*oheight; y++ ) {
      #pragma simd
      for (int x = 0; x < owidth; x++) {
         x_new[y*opitch+x] += dx[y*opitch+x];
      }
    }
    //float diff = 0.0f;
    //for (int  j = 0; j < oheight; j++) for (int k = 0; k < owidth; k++) diff += fabs(warped_image[j*opitch+k] - first_small_image[j*opitch+k]);
    //printf("Diff = %f \n", diff/(owidth*oheight));
    
    float* tmp = x_old; x_old = x_new; x_new = tmp;
    iwidth = owidth; iheight = oheight; ipitch = opitch;
    //printf("Single scale total compute time = %f seconds \n", timestamp() - ts1);
  }

  #pragma omp parallel for num_threads(nthreads)
  for (int j = 0; j < 2*pitch*height; j++) x[j] = x_old[j];
 
  delete [] dx;
  delete [] warped_image;
  delete [] first_small_image;
  delete [] second_small_image;
  delete [] x_old;
  delete [] x_new;
  
  printf("Single scale solve time (cumulative): %f seconds\n", ssof_time);
  }


}


void write_middlebury(const float* flow, int width, int height, int pitch) {
  FILE * flofile = fopen("out.flo", "wb");
  if(!flofile)
  {
    std::cout << "Could not open out.flo" << std::endl;
  }
  fprintf(flofile, "PIEH");
  assert(fwrite(&(width), sizeof(int),1, flofile));
  assert(fwrite(&(height), sizeof(int),1, flofile));
  // write the data
  for (int y = 0; y < height; y++)
    for (int x = 0; x < width; x++) {
      float u = flow[x + y * pitch];
      float v = flow[x + y * pitch + pitch * height];
      fwrite(&u,sizeof(float),1,flofile);
      fwrite(&v,sizeof(float),1,flofile);
    }
  fclose(flofile);
#define DEBUG
#ifdef DEBUG
checksum(flow, width, height, pitch);
#endif
}


int main(int argc, char* argv[])
{
  int c;
  char *image_fname_prefix = NULL;
  char *image_fname_suffix = NULL;
  int nframes = 2;
  char *second_image_fname = NULL;
  float lam = 0.025;
  nthreads = 1;

  if (argc < 7) {
    printf("usage: %s -f <image_name_prefix> -s <image_name_suffix> -n <number of frames, min:2, max:99> -l <lambda (optional)>\n", argv[0]);
    exit(1);
  }

  while((c = getopt(argc, argv, "f:s:n:l:"))!=-1)
  {
    switch(c)
    {
	case 'f':
          image_fname_prefix = optarg;
	  break;
	case 's':
          image_fname_suffix = optarg;
	  break;
	case 'n':
          nframes = atoi(optarg);
          if (nframes < 2 || nframes >= 100) { 
            printf("Insufficient/Too many frames (min 2, max 99)\n");
            exit(1); 
          }
	  break;
	case 'l':
          lam = atof(optarg);
	  break;
    }
  }


  char** image_data = new char*[nframes];
  float** float_image_data = new float*[nframes];
  float** flow = new float*[nframes-1];
  int width, height, pitch;

  for (int  nf = 0; nf < nframes; nf++) {
    char filename[512];
    sprintf(filename, "%s%02d%s", image_fname_prefix, nf+1, image_fname_suffix);
 
    std::ifstream f(filename, std::ifstream::binary);
    f.read((char*)&width, sizeof(float));
    f.read((char*)&height, sizeof(float));
    std::cout << "Width: " << width << "\tHeight: " << height << std::endl;
    int pitch_multiple = 128;
    // Figure out padding
    size_t pitchInFloats = ((((width * sizeof(float)) + pitch_multiple - 1) / pitch_multiple) * pitch_multiple) / sizeof(float);
    pitch = pitchInFloats;
    image_data[nf] = new char[width*height];

    //char* first_image_data = new char[width*height];
    //char* second_image_data = new char[width*height];
    f.read( image_data[nf], width*height);
    f.close();

    float_image_data[nf] = new float[pitchInFloats*height];
    #pragma omp parallel for //num_threads(nthreads)
    for (int i = 0; i < height; i++) {
      for (int j = 0; j < width; j++) {
        int pp = i * pitch + j;
        int pp_ = i * width + j;
        float_image_data[nf][pp] = C2F(image_data[nf][pp_]);
        //first[pp] = C2F(first_image_data[pp_]);
        //second[pp] = C2F(second_image_data[pp_]);
      }
    }
  }
  for (int nf = 0; nf < nframes-1; nf++) {
    flow[nf] = new float[2*pitch*height];
  }

  Ix = new float[pitch*height];
  Iy = new float[pitch*height];
  It = new float[pitch*height];
  r = new float[2*pitch*height];
  z = new float[2*pitch*height];
  p = new float[2*pitch*height];
  Ap = new float[2*pitch*height];
  x = new float[2*pitch*height];

  // N.B.: nlevels corresponds to the `ns` argument passed to
  // multiScaleOpticalFlow in the Julia versions of the benchmark, or
  // the `nscales` argument passed to multi_scale_optical_flow in the
  // Python versions.
  int nlevels = (int)round(log(50.0/(float)width)/log(0.9));
  printf("#Levels = %d \n", nlevels);

  double ts = timestamp();
    
  for (int nf = 0; nf < nframes-1; nf++) {
      float* first = float_image_data[nf];
      float* second = float_image_data[nf+1];
      float* local_flow = flow[nf];
      optical_flow2(first, second, local_flow, 100, lam, width, height, pitch, nlevels);
  }

  double elapsed = timestamp() - ts;
  printf("SELFTIMED %f\n", elapsed);
  printf("Total elapsed time: %f seconds\n", elapsed);

  write_middlebury(flow[0], width, height, pitch);
  delete[] flow;
  //delete[] first;
  //delete[] second;

  delete [] Ix;
  delete [] Iy;
  delete [] It;
  delete [] r;
  delete [] z;
  delete [] p;
  delete [] Ap;
  delete [] x;
}
