#include <cassert>
#include <cmath>
#include <omp.h>
#include "common.h"

void GoBackInTime( int numPaths, int numSteps, const REALVEC* __restrict assetPtr, REAL strike, REALVEC* __restrict cashFlowPutPtr ) {
    assert(numPaths%VLEN==0);
    __assume_aligned(assetPtr,64);
    __assume_aligned(cashFlowPutPtr,64);
    const size_t numPathStrips = numPaths/VLEN;
#define asset(s,p) assetPtr[(s)*numPathStrips+(p)]
#define cashFlowPut(p) cashFlowPutPtr[p]
    REAL avgPathFactor = REAL(1.0) / REAL(numPaths);
    REAL betaPut0, betaPut1, betaPut2;
    for( int s=numSteps-1; ; --s ) {
        REAL x0=0, x1=0, x2=0, sum0=0, sum1=0, sum2=0, sum3=0, sum4=0;
#pragma omp parallel reduction(+:x0,x1,x2,sum0,sum1,sum2,sum3,sum4)
        {
            // Figure out which strips current thread computes.
            int pfirst, plast;
            GetStripInterval( pfirst, plast, numPathStrips );

            REALVEC sum0p, sum1p, sum2p, sum3p, sum4p, x0p, x1p, x2p, assetVal;
            sum0p[:] = sum1p[:] = sum2p[:] = sum3p[:] = sum4p[:] = 0;
            x0p[:] = x1p[:] = x2p[:] = 0;
            for( int p=pfirst; p<plast; ++p ) {
                assetVal[:] = asset(s+1,p)[:];
                if( s<numSteps-1 ) {
                    if( assetVal[:]<=strike ) {
                        REALVEC regImpliedCashFlow, payoff;
                        regImpliedCashFlow[:] = betaPut0 + betaPut1*assetVal[:] + betaPut2*(assetVal[:]*assetVal[:]);
                        payoff[:] = strike-assetVal[:];
                        if( payoff[:]>regImpliedCashFlow[:] )
                            cashFlowPut(p)[:] = payoff[:];
                    }
                } else {
                    cashFlowPut(p)[:] = std::max(strike - assetVal[:], REAL(0)) * avgPathFactor;
                }
                if( s>0 ) {
                    assetVal[:] = asset(s,p)[:];
                    if( assetVal[:]<=strike ) {
                        REALVEC a2;
                        a2[:] = assetVal[:]*assetVal[:];
                        sum0p[:] += 1;
                        sum1p[:] += assetVal[:];
                        sum2p[:] += a2[:];
                        sum3p[:] += a2[:]*assetVal[:];
                        sum4p[:] += a2[:]*a2[:];
                        x0p[:] += cashFlowPut(p)[:];
                        x1p[:] += cashFlowPut(p)[:]*assetVal[:];
                        x2p[:] += cashFlowPut(p)[:]*a2[:];
                    }
                }
            }
            x0 = __sec_reduce_add(x0p[:]);
            x1 = __sec_reduce_add(x1p[:]);
            x2 = __sec_reduce_add(x2p[:]);
            sum0 = __sec_reduce_add(sum0p[:]);
            sum1 = __sec_reduce_add(sum1p[:]);
            sum2 = __sec_reduce_add(sum2p[:]);
            sum3 = __sec_reduce_add(sum3p[:]);
            sum4 = __sec_reduce_add(sum4p[:]);
        }
        if( s==0 ) 
            break;
        const REAL ridgeCorrect = 0.01;
        sum0 += ridgeCorrect;
        sum1 += ridgeCorrect;
        sum2 += ridgeCorrect;
        sum3 += ridgeCorrect;
        sum4 += ridgeCorrect;
        REAL r[3][3] = {{sum0,sum1,sum2},
                        {sum1,sum2,sum3},
                        {sum2,sum3,sum4}};
        REAL invsqr[3][3];
        invert3(invsqr,r);

        betaPut0 = x0*invsqr[0][0] + x1*invsqr[1][0] + x2*invsqr[2][0];
        betaPut1 = x0*invsqr[0][1] + x1*invsqr[1][1] + x2*invsqr[2][1];
        betaPut2 = x0*invsqr[0][2] + x1*invsqr[1][2] + x2*invsqr[2][2];
    }
}
