#ifndef TIMER_HPP
#define TIMER_HPP

#include <sys/time.h>                // for gettimeofday()
#include <cstdlib>

/** Gets the time between construction and destruction and stores it in a
 * reference result variable.
 * Create an object of this class in a scope, and when it's destructed, the
 * time is put into the constructor's parameter.
 */
class ScopedTimer
{
public:
  ScopedTimer(double& res) : res_(res) 
  {
    gettimeofday(&t1_, NULL);
  }

  ~ScopedTimer()
  {
    gettimeofday(&t2_, NULL);
    res_ = (t2_.tv_sec - t1_.tv_sec) * 1000.0;
    res_ += (t2_.tv_usec - t1_.tv_usec) / 1000.0;
  }
private:
  double& res_;
  timeval t1_, t2_;
};

class Timer
{
  public:
  Timer() { start(); }
  void start() { gettimeofday(&t1_, NULL); }
  void stop()  { gettimeofday(&t2_,NULL);  }
  double getTime() {
    double res = (t2_.tv_sec - t1_.tv_sec) * 1000.0;
    res += (t2_.tv_usec - t1_.tv_usec) / 1000.0;
    return res;
  }
private:
  timeval t1_, t2_;
};



#endif
