#include <omp.h>
#include <cassert>
#include "common.h"
#include "Random.h"
#include "params.h"
 
void WarmUp( int numPaths, int numSteps, REALVEC* __restrict assetPtr, REALVEC* __restrict cashFlowPutPtr ) {
    assert(assetPtr);
    assert(numPaths%VLEN==0);
    __assume_aligned(assetPtr,64);
    __assume_aligned(cashFlowPutPtr,64);
    const size_t numPathStrips = numPaths/VLEN;
#define asset(s,p) assetPtr[(s)*numPathStrips+(p)]
#define cashFlowPut(p) cashFlowPutPtr[p]
 
    REALVEC* samplePtr = (REALVEC*) _mm_malloc(omp_get_max_threads()*numSteps*sizeof(REALVEC),64);

    // FIXME - figure out right way to parameterize seed    
    int seed = 12345;

#pragma omp parallel 
    {
        // Figure out on which strips current thread operates.
        int pfirst, plast;
        int id = GetStripInterval( pfirst, plast, numPathStrips );
        REALVEC* sample = samplePtr + id*numSteps;

        // Initialize random number generator on per-thread basis
        VSLStreamStatePtr stream;
        CheckVSL( vslNewStream(&stream, VSL_BRNG_MRG32K3A, seed) );
        CheckVSL( vslSkipAheadStream(stream, pfirst*numSteps*VLEN) );  
        REAL mean = 0;
        REAL deviation = 1;
        genGaussian(stream, numSteps*VLEN, (REAL*)sample, mean, deviation );
        vslDeleteStream(&stream);

        // Page in memory for asset and cashFlowPut 
        for( int p=pfirst; p<plast; ++p ) {
            cashFlowPut(p)[:] = 0;
            for( int s=0; s<=numSteps; ++s ) 
                asset(s,p)[:] = 0;
        }
    }
    _mm_free(samplePtr);
}

