#include <cmath>
#include <algorithm>
#include <iostream>
#include "params.h"

const double FUZZ = 0.00000001;

inline REAL det3(REAL a, REAL b, REAL c,
                 REAL d, REAL e, REAL f,
                 REAL g, REAL h, REAL i) { 
    return a*e*i + d*h*c + g*b*f - g*e*c - d*b*i - a*h*f; 
}

// Determinant of a 3x3 matrix (for inversion)
inline REAL determinant(REAL x[3][3]) {
    REAL det;

    det = det3(x[0][0], x[0][1], x[0][2],
               x[1][0], x[1][1], x[1][2],
               x[2][0], x[2][1], x[2][2]);
    return det;
}

static void invert3(REAL a[3][3], REAL x[3][3]) {
    REAL det  = determinant(x);
    if(std::abs(det)<FUZZ)
        std::cout << "Trying to invert non invertible matrix" << std::endl;

    a[0][0] = (x[1][1]*x[2][2] - x[2][1]*x[1][2])/det;
    a[0][1] = (x[0][2]*x[2][1] - x[0][1]*x[2][2])/det;
    a[0][2] = (x[0][1]*x[1][2] - x[0][2]*x[1][1])/det;
    a[1][0] = (x[1][2]*x[2][0] - x[1][0]*x[2][2])/det;
    a[1][1] = (x[0][0]*x[2][2] - x[0][2]*x[2][0])/det;
    a[1][2] = (x[0][2]*x[1][0] - x[0][0]*x[1][2])/det;
    a[2][0] = (x[1][0]*x[2][1] - x[1][1]*x[2][0])/det;
    a[2][1] = (x[0][1]*x[2][0] - x[0][0]*x[2][1])/det;
    a[2][2] = (x[0][0]*x[1][1] - x[0][1]*x[1][0])/det;
}

const size_t VLEN = 8;

typedef REAL REALVEC[VLEN];

inline int GetStripInterval( int& pfirst, int& plast, size_t numPathStrips ) {
    // Figure out which strips current thread computes.
    int nthread = omp_get_num_threads();
    size_t chunksize = (numPathStrips+nthread-1)/nthread;
    int id = omp_get_thread_num();
    pfirst = id*chunksize;
    plast = std::min(pfirst+chunksize,numPathStrips);
    return id;
}

void WarmUp( int numPaths, int numSteps, REALVEC* __restrict assetPtr, REALVEC* __restrict cashFlowPutPtr );
void InitAssets( int numPaths, int numSteps, REALVEC* __restrict assetPtr, REAL spot, REAL vol, REAL maturity );
void GoBackInTime( int numPaths, int numSteps, const REALVEC* __restrict assetPtr, REAL strike, REALVEC* __restrict cashFlowPutPtr );
