#!/usr/bin/env python

# Produces legend.pdf to go in the paper.

from collections import OrderedDict
import matplotlib as mpl
mpl.use('Agg')
import matplotlib.patches as mpatches
import matplotlib.pyplot as plt

fig, ax = plt.subplots(figsize=(19,2))

# Manually create legend patches
color_dict = OrderedDict([
    ("Python",                           '#f89bd4'),
    ("Matlab (1 thread)",                '#fc8d62'),
    ("Matlab",                           '#b84920'),
    ("Julia (1 thread)",                 '#ccebc5'),
    ("Julia (explicit loops, 1 thread)", '#88a781'),
    ("@acc (1 thread)",                  '#4c9cdf'),
    ("@acc (36 threads)",                '#08589e'),
    ("C (simple serial, 1 thread)",      '#a784e6'),
    ("C/C++ (optimized, 36 threads)",    '#6340a2')])

patches = []
for i,label in enumerate(color_dict):
    patches.insert(i, mpatches.Patch(color=color_dict[label], label=label))

plt.legend(handles=patches, loc="center", mode="expand", ncol=3, prop={'size':20})

# Hide everything else
ax.axis('off')

plt.savefig("legend.pdf")
