#!/usr/bin/env python

# Produces opts_legend.pdf to go in the paper.

from collections import OrderedDict
import matplotlib as mpl
mpl.use('Agg')
import matplotlib.patches as mpatches
import matplotlib.pyplot as plt

fig, ax = plt.subplots(figsize=(16,1))

# Manually create legend patches
color_dict = OrderedDict([
    ("Julia (1 thread)",                  '#ccebc5'), # Same as in main results
    ("@acc (1 thread, no optimizations)", '#a8ddb5'),
    ("+Parallelization",                  '#7bccc4'),
    ("+Fusion",                           '#4eb3d3'),
    ("+Hoisting",                         '#2b8cbe'),
    ("+Vectorization",                    '#08589e')]) # Same as "@acc (36 threads)" in main results

patches = []
for i,label in enumerate(color_dict):
    patches.insert(i, mpatches.Patch(color=color_dict[label], label=label))

plt.legend(handles=patches, loc="center", mode="expand", ncol=3, prop={'size':20})

# Hide everything else
ax.axis('off')

plt.savefig("opts-legend.pdf")
