#!/bin/bash

# Use this script to run benchmarks and produce reports.  It takes a
# file containing a list of benchmark identifiers as its argument.

# Usage: ./run-benchmarks.sh benchmark-list.txt

source ${BASH_SOURCE%/*}/env-vars.sh

# Before running benchmarks, make sure necessary packages are
# installed and that everything is updated and built.
for julia_version in $JULIA_0_5_EXE; do
    # Clean up after old runs so that tree won't be dirty
    rm -f $JULIA_PKGDIR/v0.5/ParallelAccelerator/deps/a.out

    # Install/update/build packages
    $julia_version -e 'Pkg.update()'
    $julia_version -e 'Pkg.add("DocOpt"); Pkg.add("Match"); Pkg.add("Images"); Pkg.add("ImageMagick"); Pkg.add("MAT")'
    $julia_version -e 'Pkg.add("ParallelAccelerator"); Pkg.add("CompilerTools")'
    $julia_version -e 'Pkg.checkout("ParallelAccelerator"); Pkg.checkout("CompilerTools")'
    yes | $julia_version -e 'Pkg.build("ParallelAccelerator")'
done

DATE=`date --rfc-3339=date`

while IFS='' read -r benchmark || [[ -n "$benchmark" ]]
do
    echo "Running on benchmark $benchmark"

    # Create temporary directories to store results.
    mkdir -p /tmp/results/$benchmark
    
    BENCHMARK_DIR=${BASH_SOURCE%/*}/../$benchmark/benchmarking
    RESULTS_DIR=/tmp/results/$benchmark
    LOG1=totaltimes-${DATE}.log
    LOG2=offloadtimes-${DATE}.log
    REPORT_LOG=benchmarking-report-${DATE}.log
    OPTS_LOG=optimizations-${DATE}.log

    cd $BENCHMARK_DIR && ./benchmark.sh 1> $RESULTS_DIR/$LOG1 2> $RESULTS_DIR/$LOG2 \
        && ../../util/munge-results.py $RESULTS_DIR/$LOG1 $RESULTS_DIR/$LOG2 --filename-suffix=$DATE > $RESULTS_DIR/$REPORT_LOG \
        && cat $RESULTS_DIR/$REPORT_LOG
    cd -

    # Benchmark @acc versions with various optimizations enabled/disabled.
    $JULIA_0_5_EXE ${BASH_SOURCE%/*}/configtest.jl --benchmarks-dir=${BASH_SOURCE%/*}/.. --workloads=$benchmark > $RESULTS_DIR/$OPTS_LOG \
        && ${BASH_SOURCE%/*}/munge-results.py $RESULTS_DIR/$LOG1 $RESULTS_DIR/$LOG2 --opts-log=$RESULTS_DIR/$OPTS_LOG --filename-suffix=$DATE
done < "$1"

# Produce legends.
python ${BASH_SOURCE%/*}/opts-legend.py && mv ${BASH_SOURCE%/*}/opts-legend.pdf /tmp/results/
python ${BASH_SOURCE%/*}/legend.py && mv ${BASH_SOURCE%/*}/legend.pdf /tmp/results/
